package com.btc.ep.base.dal.tests.it;

import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.btc.ep.core.Services;
import com.btc.ep.dal.persist.PersistenceService2;

import optimisticlockwithremove.dmo.Architecture;
import optimisticlockwithremove.dmo.FormalSpecification;
import optimisticlockwithremove.dmo.Formalization;
import optimisticlockwithremove.dmo.InterfaceObject;
import optimisticlockwithremove.dmo.Mapping;
import optimisticlockwithremove.dmo.PatternParameter;
import optimisticlockwithremove.dmo.RequirementReference;
import optimisticlockwithremove.dmo.Scope;
import optimisticlockwithremove.dmo.impl.ArchitectureImpl;
import optimisticlockwithremove.dmo.impl.FormalRequirementImpl;
import optimisticlockwithremove.dmo.impl.FormalSpecificationImpl;
import optimisticlockwithremove.dmo.impl.InterfaceObjectImpl;
import optimisticlockwithremove.dmo.impl.MappingImpl;
import optimisticlockwithremove.dmo.impl.PatternParameterImpl;
import optimisticlockwithremove.dmo.impl.RequirementReferenceImpl;
import optimisticlockwithremove.dmo.impl.ScopeImpl;
import optimisticlockwithremove.dmo.impl.SimplifiedUniversalPatternImpl;
import optimisticlockwithremove.dmo.impl.SimulinkScopeImpl;

public class QueryCollection {

	private static PersistenceService2 persistenceService2;

	@Before
	public void setUp() throws Exception {
		deleteDatabase();

		InterfaceObjectImpl io1 = new InterfaceObjectImpl();
		io1.setName("io1");

		InterfaceObjectImpl io2 = new InterfaceObjectImpl();
		io2.setName("io2");
		
		InterfaceObjectImpl io3 = new InterfaceObjectImpl();
		io3.setName("io3");
		
		ScopeImpl scope = new ScopeImpl();
		scope.addInterfaceObject(io1);
		scope.addInterfaceObject(io2);
		scope.addInterfaceObject(io3);
		io1.setScope(scope);
		io2.setScope(scope);
		
		ArchitectureImpl architecture = new ArchitectureImpl();
		architecture.addRootScope(scope);
		scope.setArchitecture(architecture);
		architecture.setIsMaster(true);
		getPersistenceService2().persist(architecture);
		
		Mapping mapping = new MappingImpl();
		mapping.addValue(io1);
		getPersistenceService2().persist(mapping);

		getPersistenceService2().commit();
		getPersistenceService2().close();
	}

	@Test
	public void query() {

		getPersistenceService2().open();

		String queryString = "SELECT io FROM " + InterfaceObjectImpl.class.getName() + " io, "  + ScopeImpl.class.getName() 
				+ " s, " + ArchitectureImpl.class.getName() + " a "
				+ "where io.scope = s and s.architecture = a and a.isMaster = true";
		String formattedQueryString = String.format(queryString, InterfaceObjectImpl.class.getName());
		List<InterfaceObject> list = getPersistenceService2().getModelElementsByQuery(formattedQueryString, InterfaceObject.class);
		
		for (InterfaceObject interfaceObject : list) {
			System.out.println("io: " + interfaceObject.getName() + " (" + list.size() + ")");
		}

		queryString = "SELECT io FROM " + InterfaceObjectImpl.class.getName() + " io, " + MappingImpl.class.getName() 
				+ " m WHERE io.scope.architecture.isMaster = true and io not member of m.values";
		formattedQueryString = String.format(queryString, InterfaceObjectImpl.class.getName());
		list = getPersistenceService2().getModelElementsByQuery(formattedQueryString, InterfaceObject.class);
		
		for (InterfaceObject interfaceObject : list) {
			System.out.println("io: " + interfaceObject.getName() + " (" + list.size() + ")");
		}

		getPersistenceService2().commit();
	}

	protected void deleteDatabase() {
		getPersistenceService2().updateQuery("DELETE FROM Object");
	}

	public PersistenceService2 getPersistenceService2() {
		if (persistenceService2 == null) {
			persistenceService2 = Services.get(PersistenceService2.class);
		}
		return persistenceService2;
	}

}
