package com.btc.ep.base.dal.tests.it;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import com.btc.ep.core.Services;
import com.btc.ep.dal.persist.PersistenceService2;

import optimisticlockwithremove.dmo.FormalAssumption;
import optimisticlockwithremove.dmo.FormalRequirement;
import optimisticlockwithremove.dmo.FormalSpecification;
import optimisticlockwithremove.dmo.Formalization;
import optimisticlockwithremove.dmo.PatternParameter;
import optimisticlockwithremove.dmo.RequirementReference;
import optimisticlockwithremove.dmo.Scope;
import optimisticlockwithremove.dmo.impl.FormalAssumptionImpl;
import optimisticlockwithremove.dmo.impl.FormalRequirementImpl;
import optimisticlockwithremove.dmo.impl.FormalSpecificationImpl;
import optimisticlockwithremove.dmo.impl.PatternParameterImpl;
import optimisticlockwithremove.dmo.impl.RequirementReferenceImpl;
import optimisticlockwithremove.dmo.impl.ScopeImpl;
import optimisticlockwithremove.dmo.impl.SimplifiedUniversalPatternImpl;
import optimisticlockwithremove.dmo.impl.SimulinkScopeImpl;

public class RemovedFormalRequirementIsAlwaysInTheDatabaseTest {

	private static PersistenceService2 persistenceService2;

	static String scopeID;
    static String formalSpecificationID;
    
	@Before
	public void setUp() throws Exception {
		deleteDatabase();

        Scope scope = new SimulinkScopeImpl();
        getPersistenceService2().persist(scope);
        scopeID = scope.getUid();

        SimplifiedUniversalPatternImpl pattern = new SimplifiedUniversalPatternImpl();
        getPersistenceService2().persist(pattern);

        FormalRequirementImpl formalSpecification = new FormalRequirementImpl("", scopeID, pattern);
        getPersistenceService2().persist(formalSpecification);
        formalSpecificationID = formalSpecification.getUid();
        
        FormalAssumption formalAssumption = new FormalAssumptionImpl("assumption", scopeID, pattern);
        formalSpecification.getAssumptions().add(formalAssumption);
        
        getPersistenceService2().commit();
        getPersistenceService2().close();
	}
    
    @Test
    public void remove() {
    	
    	getPersistenceService2().open();

        Scope scope = getPersistenceService2().find(ScopeImpl.class, scopeID);

        FormalRequirement formalSpecification = getPersistenceService2().find(FormalRequirementImpl.class, formalSpecificationID);
        Formalization pattern = formalSpecification.getFormalization();
        
        formalSpecification.getAssumptions().remove(formalSpecification.getAssumptions().toArray(new FormalAssumptionImpl[1])[0]);

        pattern.cleanAllObservers();

        String queryString =
            "SELECT patternParameter FROM %s AS patternParameter JOIN patternParameter.pattern AS pattern "
                + "WHERE pattern.uid = ?1";
        String formattedQueryString = String.format(queryString, PatternParameterImpl.class.getName());
        getPersistenceService2().getModelElementsByQuery(formattedQueryString, PatternParameter.class, pattern.getUid());
        
        List<FormalRequirementImpl> list =
        		getPersistenceService2().getModelElementsByQuery(
        				"select f from FormalRequirementImpl f JOIN f.formalization p where p.uid = ?1",
                FormalRequirementImpl.class, pattern.getUid());

        getPersistenceService2().remove(pattern);

        String query = String.format("SELECT reference FROM %s as reference where reference.refElementID IN ?1",
            RequirementReferenceImpl.class.getName());
        List<RequirementReferenceImpl> listByQuery = getPersistenceService2().getModelElementsByQuery(
        		query, RequirementReferenceImpl.class, formalSpecification.getUid());
        
        for (RequirementReference ref : listByQuery) {
        	getPersistenceService2().remove(ref);
        }
        
        String formalSpecId = formalSpecification.getUid();
        getPersistenceService2().remove(formalSpecification);
        getPersistenceService2().commit();
        
        
    	getPersistenceService2().open();

        formalSpecification = getPersistenceService2().find(FormalRequirementImpl.class, formalSpecId);
        Assert.assertNull(formalSpecification);
        getPersistenceService2().commit();
    }

	protected void deleteDatabase() {
		getPersistenceService2().updateQuery("DELETE FROM Object");
	}

	public PersistenceService2 getPersistenceService2() {
		if (persistenceService2 == null) {
			persistenceService2 = Services.get(PersistenceService2.class);
		}
		return persistenceService2;
	}
    
}
