package optimisticlockwithremove.dmo.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.Access;
import javax.persistence.AccessType;
import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.ManyToOne;

import optimisticlockwithremove.dmo.Requirement;

@Entity
@Access (AccessType.FIELD)
public class RequirementImpl extends DescribableElementImpl implements Requirement {

    @Basic
    private String externalId;

    @Basic
    private String url;

    @Basic
    private String identifier;

    @Basic
    private boolean removed;

    @Basic
    private String outdatedName;

    @Basic
    private String outdatedDescription;

    @ManyToOne (targetEntity = RequirementImpl.class,
        cascade = {CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
    private Requirement parentReq;

    @Basic
    private String scopeID;

    @ElementCollection
    private Map<String, String> userDefinedValues = new HashMap<>();

    @Basic
    private Date dateOfLastUpdate;

    protected RequirementImpl() {
        scopeID = null;
    }

    public RequirementImpl(String externalID) {
        this();
        this.externalId = externalID;
    }

    @Override
    public String getIdentifier() {
        return identifier;
    }

    public void setIdentifier(String identifier) {
        this.identifier = identifier;
    }

    @Override
    public String getURL() {
        return url;
    }

    public void setURL(String url) {
        this.url = url;
    }

    @Override
    public boolean isRemoved() {
        return removed;
    }

    public void setRemoved(boolean removed) {
        this.removed = removed;
    }

    public void setDescription(String description) {
        String oldDesc = getDescription();
        if (!oldDesc.equals("")) {
            this.outdatedDescription = oldDesc;
        }
        super.setDescription(description);
    }

    public void setName(String name) {
        String oldName = getName();
        if (!oldName.equals("")) {
            this.outdatedName = oldName;
        }
        super.setName(name);
    }

    @Override
    public String getOutdatedName() {
        return this.outdatedName;
    }

    @Override
    public String getOutdatedDescription() {
        return this.outdatedDescription;
    }

    @Override
    public String getExternalId() {
        return this.externalId;
    }

    @Override
    public Requirement getParent() {
        return this.parentReq;
    }

    public void setParent(Requirement req) {
        this.parentReq = req;
    }

    @Override
    public String getScopeID() {
        return scopeID;
    }

    public void setScopeID(String scopeID) {
        this.scopeID = scopeID;
    }

    public void setAttributeValue(String key, String value) {
        this.userDefinedValues.put(key, value);
    }

    public void setDateOfLastUpdate(Date dateOfLastUpdate) {
        this.dateOfLastUpdate = dateOfLastUpdate;
    }

    @Override
    public Date getDateOfLastUpdate() {
        return this.dateOfLastUpdate;
    }
}